@@ -4,6 +4,8 @@
  * Copyright (C) 2003, 2004, 2005 Red Hat, Inc.
  * Copyright (C) 2005 Nokia
  *
+ * SPDX-License-Identifier: AFL-2.1 OR GPL-2.0-or-later
+ *
  * Licensed under the Academic Free License version 2.1
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -33,7 +35,6 @@
 #include "dbus-gvalue-utils.h"
 #include "dbus-gobject.h"
 #include <string.h>
-#include <glib/gi18n.h>
 #include <gobject/gvaluecollector.h>
 #include <gio/gio.h>
 
@@ -677,7 +678,7 @@ unassociate_proxies (gpointer key, gpointer val, gpointer user_data)
 
       manager = priv->manager;
 
-      if (!strcmp (priv->name, name))
+      if (priv->name != NULL && !strcmp (priv->name, name))
 	{
 	  if (!priv->for_owner)
 	    {
@@ -1090,10 +1091,12 @@ dbus_g_proxy_manager_unregister (DBusGProxyManager *manager,
 	    }
 	  else
 	    {
-	      link = g_slist_find (manager->unassociated_proxies, proxy);
-	      g_assert (link != NULL);
-
-	      manager->unassociated_proxies = g_slist_delete_link (manager->unassociated_proxies, link);
+              link = g_slist_find (manager->unassociated_proxies, proxy);
+              if (link != NULL)
+                {
+                  manager->unassociated_proxies = g_slist_delete_link (
+                      manager->unassociated_proxies, link);
+                }
 	    }
 	}
       else
@@ -1138,7 +1141,8 @@ dbus_g_proxy_manager_unregister (DBusGProxyManager *manager,
       manager->proxy_lists = NULL;
     }
 
-  if (g_hash_table_size (manager->owner_match_rules) == 0)
+  if (manager->owner_match_rules != NULL &&
+      g_hash_table_size (manager->owner_match_rules) == 0)
     {
       g_hash_table_destroy (manager->owner_match_rules);
       manager->owner_match_rules = NULL;
@@ -1248,8 +1252,11 @@ dbus_g_proxy_manager_filter (DBusConnection    *connection,
       GSList *tmp;
       const char *sender;
 
+      sender = dbus_message_get_sender (message);
+
       /* First we handle NameOwnerChanged internally */
-      if (dbus_message_is_signal (message,
+      if (g_strcmp0 (sender, DBUS_SERVICE_DBUS) == 0 &&
+	  dbus_message_is_signal (message,
 				  DBUS_INTERFACE_DBUS,
 				  "NameOwnerChanged"))
 	{
@@ -1278,8 +1285,6 @@ dbus_g_proxy_manager_filter (DBusConnection    *connection,
 	    }
 	}
 
-      sender = dbus_message_get_sender (message);
-
       /* dbus spec requires these, libdbus validates */
       g_assert (dbus_message_get_path (message) != NULL);
       g_assert (dbus_message_get_interface (message) != NULL);
@@ -1493,6 +1498,12 @@ dbus_g_proxy_class_init (DBusGProxyClass *klass)
   object_class->dispose = dbus_g_proxy_dispose;
   object_class->constructor = dbus_g_proxy_constructor;
   
+/**
+ * DBusGProxy::destroy:
+ * @dbusgproxy: the object which received the signal.
+ *
+ * Deprecated: New code should use #GDBusProxy instead.
+ */
   signals[DESTROY] =
     g_signal_new ("destroy",
 		  G_OBJECT_CLASS_TYPE (object_class),
@@ -1502,6 +1513,12 @@ dbus_g_proxy_class_init (DBusGProxyClass *klass)
                   g_cclosure_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
 
+/*
+ * DBusGProxy::received:
+ * @dbusgproxy: the object which received the signal.
+ *
+ * Deprecated: New code should use #GDBusProxy instead.
+ */
   signals[RECEIVED] =
     g_signal_new ("received",
 		  G_OBJECT_CLASS_TYPE (object_class),
@@ -1688,7 +1705,6 @@ marshal_dbus_message_to_g_marshaller (GClosure     *closure,
    */
 #define MAX_SIGNATURE_ARGS 20
   GValueArray *value_array;
-  GSignalCMarshaller c_marshaller;
   DBusGProxy *proxy;
   DBusMessage *message;
   GArray *gsignature;
@@ -1707,11 +1723,6 @@ marshal_dbus_message_to_g_marshaller (GClosure     *closure,
 
   priv = DBUS_G_PROXY_GET_PRIVATE(proxy);
 
-  c_marshaller = _dbus_gobject_lookup_marshaller (G_TYPE_NONE, gsignature->len,
-						  (GType*) gsignature->data);
-
-  g_return_if_fail (c_marshaller != NULL);
-  
   {
     DBusGValueMarshalCtx context;
     context.recursion_depth = 0;
@@ -1730,9 +1741,9 @@ marshal_dbus_message_to_g_marshaller (GClosure     *closure,
   g_value_init (g_value_array_get_nth (value_array, 0), G_TYPE_FROM_INSTANCE (proxy));
   g_value_set_instance (g_value_array_get_nth (value_array, 0), proxy);
 
-  (* c_marshaller) (closure, return_value, value_array->n_values,
-		    value_array->values, invocation_hint, marshal_data);
-  
+  g_cclosure_marshal_generic (closure, return_value, value_array->n_values,
+      value_array->values, invocation_hint, marshal_data);
+
   g_value_array_free (value_array);
 }
 
@@ -1805,6 +1816,20 @@ dbus_g_proxy_emit_remote_signal (DBusGProxy  *proxy,
   goto out;
 }
 
+/**
+ * DBusGProxyCallNotify:
+ * @proxy: the proxy on which the method was called
+ * @call_id: the call in progress
+ * @user_data: data passed to dbus_g_proxy_begin_call() or similar
+ *
+ * Called when a reply to the call represented by @call_id arrives.
+ * Use dbus_g_proxy_end_call() to see whether @call_id succeeded or
+ * failed, and get the arguments returned (if any) on success.
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is the standard #GAsyncReadyCallback mechanism.
+ */
+
 typedef struct
 {
   DBusGProxy *proxy;
@@ -1833,26 +1858,40 @@ d_pending_call_free (void *data)
 
   g_free (closure);
 }
-  
+
 #define DBUS_G_VALUE_ARRAY_COLLECT_ALL(VALARRAY, FIRST_ARG_TYPE, ARGS) \
-do { \
+G_STMT_START { \
   GType valtype; \
-  int i = 0; \
+  guint i = 0; \
+  \
   VALARRAY = g_value_array_new (6); \
   valtype = FIRST_ARG_TYPE; \
+  \
   while (valtype != G_TYPE_INVALID) \
     { \
-      const char *collect_err; \
+      gchar *collect_err; \
       GValue *val; \
+      \
       g_value_array_append (VALARRAY, NULL); \
       val = g_value_array_get_nth (VALARRAY, i); \
       g_value_init (val, valtype); \
       collect_err = NULL; \
       G_VALUE_COLLECT (val, ARGS, G_VALUE_NOCOPY_CONTENTS, &collect_err); \
+      \
+      if (collect_err) \
+        { \
+          g_critical ("%s: unable to collect argument %u: %s", \
+              G_STRFUNC, i, collect_err); \
+          g_free (collect_err); \
+          g_value_array_free (VALARRAY); \
+          VALARRAY = NULL; \
+          break; \
+        } \
+      \
       valtype = va_arg (ARGS, GType); \
       i++; \
     } \
-} while (0)
+} G_STMT_END
 
 DBusGProxyCall *
 manager_begin_bus_call (DBusGProxyManager    *manager,
@@ -1863,7 +1902,7 @@ manager_begin_bus_call (DBusGProxyManager    *manager,
 			GType                 first_arg_type,
 			...)
 {
-  DBusGProxyCall *call;
+  guint call_id = 0;
   DBusGProxyPrivate *priv;
   va_list args;
   GValueArray *arg_values;
@@ -1882,23 +1921,50 @@ manager_begin_bus_call (DBusGProxyManager    *manager,
     }
 
   DBUS_G_VALUE_ARRAY_COLLECT_ALL (arg_values, first_arg_type, args);
-  
-  call = DBUS_G_PROXY_ID_TO_CALL (dbus_g_proxy_begin_call_internal (manager->bus_proxy, method, notify, user_data, destroy, arg_values,-1));
 
-  g_value_array_free (arg_values);
+  if (arg_values != NULL)
+    {
+      call_id = dbus_g_proxy_begin_call_internal (manager->bus_proxy, method,
+          notify, user_data, destroy, arg_values, -1);
+
+      g_value_array_free (arg_values);
+    }
 
   va_end (args);
 
-  return call;
+  return DBUS_G_PROXY_ID_TO_CALL (call_id);
 }
 
 /**
  * SECTION:dbus-gproxy
+ * @title: DBusGProxy
  * @short_description: DBus Proxy
- * @see_also: #DBusProxy
+ * @see_also: #DBusGProxy
  * @stability: Stable
  *
- * A #DBusGProxy is a boxed type abstracting a #DBusProxy.
+ * A #DBusGProxy is a #GObject representing a remote object in a D-Bus
+ * service.
+ *
+ * Deprecated: New code should use #GDBusProxy instead.
+ */
+
+/**
+ * DBusGProxy:
+ *
+ * A #GObject representing a remote object in a D-Bus service.
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is #GDBusProxy.
+ */
+
+/**
+ * DBusGProxyCall:
+ *
+ * An opaque pointer representing an asynchronous call in progress.
+ *
+ * Deprecated: New code should use GDBus instead. There is no direct
+ *  equivalent in GDBus, but the standard #GCancellable mechanism is
+ *  analogous.
  */
 
 /*
@@ -1906,6 +1972,8 @@ manager_begin_bus_call (DBusGProxyManager    *manager,
  * Standard GObject get_type() function for DBusGProxy.
  *
  * Returns: type ID for DBusGProxy class
+ *
+ * Deprecated: New code should use #GDBusProxy instead.
  */
 GType
 dbus_g_proxy_get_type (void)
@@ -1958,8 +2026,8 @@ dbus_g_proxy_new (DBusGConnection *connection,
  * dbus_g_proxy_new_for_name:
  * @connection: the connection to the remote bus
  * @name: any name on the message bus
- * @path_name: name of the object instance to call methods on
- * @interface_name: name of the interface to call methods on
+ * @path: name of the object instance to call methods on
+ * @iface: name of the interface to call methods on
  *
  * Creates a new proxy for a remote interface exported by a connection
  * on a message bus. Method calls and signal connections over this
@@ -1981,28 +2049,30 @@ dbus_g_proxy_new (DBusGConnection *connection,
  * still be alive.
  *
  * Returns: new proxy object
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is g_dbus_proxy_new_sync().
  */
 DBusGProxy*
 dbus_g_proxy_new_for_name (DBusGConnection *connection,
                            const char      *name,
-                           const char      *path_name,
-                           const char      *interface_name)
+                           const char      *path,
+                           const char      *iface)
 {
   g_return_val_if_fail (connection != NULL, NULL);
   g_return_val_if_fail (g_dbus_is_name (name), NULL);
-  g_return_val_if_fail (g_variant_is_object_path (path_name), NULL);
-  g_return_val_if_fail (g_dbus_is_interface_name (interface_name), NULL);
+  g_return_val_if_fail (g_variant_is_object_path (path), NULL);
+  g_return_val_if_fail (g_dbus_is_interface_name (iface), NULL);
 
-  return dbus_g_proxy_new (connection, name,
-			   path_name, interface_name);
+  return dbus_g_proxy_new (connection, name, path, iface);
 }
 
 /**
  * dbus_g_proxy_new_for_name_owner:
  * @connection: the connection to the remote bus
  * @name: any name on the message bus
- * @path_name: name of the object inside the service to call methods on
- * @interface_name: name of the interface to call methods on
+ * @path: name of the object inside the service to call methods on
+ * @iface: name of the interface to call methods on
  * @error: return location for an error
  *
  * Similar to dbus_g_proxy_new_for_name(), but makes a round-trip
@@ -2020,13 +2090,17 @@ dbus_g_proxy_new_for_name (DBusGConnection *connection,
  * dbus_g_proxy_new_for_name_owner() will bind to the unique name
  * of that owner rather than the generic name.
  * 
- * Returns: new proxy object, or #NULL on error
+ * Returns: new proxy object, or %NULL on error
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is g_dbus_proxy_new_sync() with the name owner's unique name
+ *  passed as @name.
  */
 DBusGProxy*
 dbus_g_proxy_new_for_name_owner (DBusGConnection          *connection,
                                  const char               *name,
-                                 const char               *path_name,
-                                 const char               *interface_name,
+                                 const char               *path,
+                                 const char               *iface,
                                  GError                  **error)
 {
   DBusGProxy *proxy;
@@ -2034,14 +2108,13 @@ dbus_g_proxy_new_for_name_owner (DBusGConnection          *connection,
 
   g_return_val_if_fail (connection != NULL, NULL);
   g_return_val_if_fail (g_dbus_is_name (name), NULL);
-  g_return_val_if_fail (g_variant_is_object_path (path_name), NULL);
-  g_return_val_if_fail (g_dbus_is_interface_name (interface_name), NULL);
+  g_return_val_if_fail (g_variant_is_object_path (path), NULL);
+  g_return_val_if_fail (g_dbus_is_interface_name (iface), NULL);
 
   if (!(unique_name = get_name_owner (DBUS_CONNECTION_FROM_G_CONNECTION (connection), name, error)))
     return NULL;
 
-  proxy = dbus_g_proxy_new (connection, unique_name,
-			    path_name, interface_name);
+  proxy = dbus_g_proxy_new (connection, unique_name, path, iface);
   g_free (unique_name);
   return proxy;
 }
@@ -2049,43 +2122,46 @@ dbus_g_proxy_new_for_name_owner (DBusGConnection          *connection,
 /**
  * dbus_g_proxy_new_from_proxy:
  * @proxy: the proxy to use as a template
+ * @iface: name of the interface to call methods on
  * @path: of the object inside the peer to call methods on
- * @interface: name of the interface to call methods on
  *
  * Creates a proxy using an existing proxy as a template, substituting
  * the specified interface and path.  Either or both may be NULL.
  *
  * Returns: new proxy object
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is g_dbus_proxy_new_sync().
  */
 DBusGProxy*
 dbus_g_proxy_new_from_proxy (DBusGProxy        *proxy,
-			     const char        *interface,
+			     const char        *iface,
 			     const char        *path)
 {
   DBusGProxyPrivate *priv;
 
   g_return_val_if_fail (DBUS_IS_G_PROXY (proxy), NULL);
   g_return_val_if_fail (path == NULL || g_variant_is_object_path (path), NULL);
-  g_return_val_if_fail (interface == NULL ||
-      g_dbus_is_interface_name (interface), NULL);
+  g_return_val_if_fail (iface == NULL ||
+      g_dbus_is_interface_name (iface), NULL);
 
   priv = DBUS_G_PROXY_GET_PRIVATE(proxy);
   
-  if (interface == NULL)
-    interface = priv->interface;
+  if (iface == NULL)
+    iface = priv->interface;
   if (path == NULL)
     path = priv->path;
 
   return dbus_g_proxy_new (DBUS_G_CONNECTION_FROM_CONNECTION (priv->manager->connection),
 			   priv->name,
-			   path, interface);
+			   path, iface);
 }
 
 /**
  * dbus_g_proxy_new_for_peer:
  * @connection: the connection to the peer
- * @path_name: name of the object inside the peer to call methods on
- * @interface_name: name of the interface to call methods on
+ * @path: name of the object inside the peer to call methods on
+ * @iface: name of the interface to call methods on
  *
  * Creates a proxy for an object in peer application (one
  * we're directly connected to). That is, this function is
@@ -2094,20 +2170,22 @@ dbus_g_proxy_new_from_proxy (DBusGProxy        *proxy,
  * applications.
  *
  * Returns: new proxy object
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is g_dbus_proxy_new_sync().
  */
 DBusGProxy*
 dbus_g_proxy_new_for_peer (DBusGConnection          *connection,
-                           const char               *path_name,
-                           const char               *interface_name)
+                           const char               *path,
+                           const char               *iface)
 {
   DBusGProxy *proxy;
   
   g_return_val_if_fail (connection != NULL, NULL);
-  g_return_val_if_fail (g_variant_is_object_path (path_name), NULL);
-  g_return_val_if_fail (g_dbus_is_interface_name (interface_name), NULL);
+  g_return_val_if_fail (g_variant_is_object_path (path), NULL);
+  g_return_val_if_fail (g_dbus_is_interface_name (iface), NULL);
 
-  proxy = dbus_g_proxy_new (connection, NULL,
-                            path_name, interface_name);
+  proxy = dbus_g_proxy_new (connection, NULL, path, iface);
 
   return proxy;
 }
@@ -2116,17 +2194,21 @@ dbus_g_proxy_new_for_peer (DBusGConnection          *connection,
  * dbus_g_proxy_get_bus_name:
  * @proxy: the proxy
  *
- * Gets the bus name a proxy is bound to (may be #NULL in some cases).
+ * Gets the bus name a proxy is bound to (may be %NULL in some cases).
  * If you created the proxy with dbus_g_proxy_new_for_name(), then
  * the name you passed to that will be returned.
  * If you created it with dbus_g_proxy_new_for_name_owner(), then the
  * unique connection name will be returned. If you created it
- * with dbus_g_proxy_new_for_peer() then #NULL will be returned.
+ * with dbus_g_proxy_new_for_peer() then %NULL will be returned.
  *
  * It is an error to call this method on a proxy that has emitted
  * the #DBusGProxy::destroy signal.
  *
  * Returns: the bus name the proxy sends messages to
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is g_dbus_proxy_get_name() or g_dbus_proxy_get_name_owner(),
+ *  depending how the proxy was created.
  */
 const char*
 dbus_g_proxy_get_bus_name (DBusGProxy        *proxy)
@@ -2145,12 +2227,15 @@ dbus_g_proxy_get_bus_name (DBusGProxy        *proxy)
  * dbus_g_proxy_get_interface:
  * @proxy: the proxy
  *
- * Gets the object interface proxy is bound to (may be #NULL in some cases).
+ * Gets the object interface proxy is bound to (may be %NULL in some cases).
  *
  * It is an error to call this method on a proxy that has emitted
  * the #DBusGProxy::destroy signal.
  *
  * Returns: an object interface 
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is g_dbus_proxy_get_interface_name().
  */
 const char*
 dbus_g_proxy_get_interface (DBusGProxy        *proxy)
@@ -2174,6 +2259,9 @@ dbus_g_proxy_get_interface (DBusGProxy        *proxy)
  *
  * It is an error to call this method on a proxy that has emitted
  * the #DBusGProxy::destroy signal.
+ *
+ * Deprecated: New code should use GDBus instead. There is no
+ *  direct equivalent for this function: construct a new proxy instead.
  */
 void
 dbus_g_proxy_set_interface (DBusGProxy        *proxy,
@@ -2196,13 +2284,17 @@ dbus_g_proxy_set_interface (DBusGProxy        *proxy,
 
 /**
  * dbus_g_proxy_get_path:
- * Gets the path this proxy is bound to
  * @proxy: the proxy
  *
+ * Gets the path this proxy is bound to
+ *
  * It is an error to call this method on a proxy that has emitted
  * the #DBusGProxy::destroy signal.
  *
  * Returns: an object path
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is g_dbus_proxy_get_object_path().
  */
 const char*
 dbus_g_proxy_get_path (DBusGProxy        *proxy)
@@ -2333,6 +2425,16 @@ dbus_g_proxy_end_call_internal (DBusGProxy        *proxy,
   DBusPendingCall *pending;
   DBusGProxyPrivate *priv = DBUS_G_PROXY_GET_PRIVATE(proxy);
 
+  if (call_id == 0)
+    {
+      /* Being disconnected is the only reason this can happen, except
+       * for programmer error; if it was programmer error, we already
+       * emitted a critical warning. */
+      g_set_error (error, DBUS_GERROR, DBUS_GERROR_DISCONNECTED,
+          "Disconnected from D-Bus (or argument error during call)");
+      return FALSE;
+    }
+
   reply = NULL;
   ret = FALSE;
   n_retvals_processed = 0;
@@ -2488,7 +2590,10 @@ dbus_g_proxy_end_call_internal (DBusGProxy        *proxy,
  * @notify: callback to be invoked when method returns
  * @user_data: user data passed to callback
  * @destroy: function called to destroy user_data
- * @first_arg_type: type of the first argument
+ * @first_arg_type: type of the first argument, or %G_TYPE_INVALID if there
+ *    are no arguments
+ * @...: first argument, followed by any further type/value pairs, followed
+ *    by %G_TYPE_INVALID
  *
  * Asynchronously invokes a method on a remote interface. The method
  * call will not be sent over the wire until the application returns
@@ -2505,6 +2610,9 @@ dbus_g_proxy_end_call_internal (DBusGProxy        *proxy,
  * since you should be able to do a call with large arguments.
  * 
  * Returns: call identifier.
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is g_dbus_proxy_call().
  */
 DBusGProxyCall *
 dbus_g_proxy_begin_call (DBusGProxy          *proxy,
@@ -2515,7 +2623,7 @@ dbus_g_proxy_begin_call (DBusGProxy          *proxy,
 			 GType                first_arg_type,
 			 ...)
 {
-  guint call_id;
+  guint call_id = 0;
   va_list args;
   GValueArray *arg_values;
   DBusGProxyPrivate *priv = DBUS_G_PROXY_GET_PRIVATE(proxy);
@@ -2527,10 +2635,14 @@ dbus_g_proxy_begin_call (DBusGProxy          *proxy,
   va_start (args, first_arg_type);
 
   DBUS_G_VALUE_ARRAY_COLLECT_ALL (arg_values, first_arg_type, args);
-  
-  call_id = dbus_g_proxy_begin_call_internal (proxy, method, notify, user_data, destroy, arg_values, priv->default_timeout);
 
-  g_value_array_free (arg_values);
+  if (arg_values != NULL)
+    {
+      call_id = dbus_g_proxy_begin_call_internal (proxy, method, notify,
+          user_data, destroy, arg_values, priv->default_timeout);
+
+      g_value_array_free (arg_values);
+    }
 
   va_end (args);
 
@@ -2545,7 +2657,10 @@ dbus_g_proxy_begin_call (DBusGProxy          *proxy,
  * @user_data: user data passed to callback
  * @destroy: function called to destroy user_data
  * @timeout: the timeout in milliseconds, or -1 to use a default
- * @first_arg_type: type of the first argument
+ * @first_arg_type: type of the first argument, or %G_TYPE_INVALID if there
+ *    are no arguments
+ * @...: first argument, followed by any further type/value pairs, followed
+ *    by %G_TYPE_INVALID
  *
  * Asynchronously invokes a method on a remote interface. The method
  * call will not be sent over the wire until the application returns
@@ -2562,6 +2677,9 @@ dbus_g_proxy_begin_call (DBusGProxy          *proxy,
  * since you should be able to do a call with large arguments.
  *
  * Returns: call identifier.
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is g_dbus_proxy_call().
  */
 DBusGProxyCall *
 dbus_g_proxy_begin_call_with_timeout (DBusGProxy          *proxy,
@@ -2573,7 +2691,7 @@ dbus_g_proxy_begin_call_with_timeout (DBusGProxy          *proxy,
                          GType                first_arg_type,
                          ...)
 {
-  guint call_id;
+  guint call_id = 0;
   va_list args;
   GValueArray *arg_values;
 
@@ -2586,9 +2704,13 @@ dbus_g_proxy_begin_call_with_timeout (DBusGProxy          *proxy,
 
   DBUS_G_VALUE_ARRAY_COLLECT_ALL (arg_values, first_arg_type, args);
 
-  call_id = dbus_g_proxy_begin_call_internal (proxy, method, notify, user_data, destroy, arg_values,timeout);
+  if (arg_values != NULL)
+    {
+      call_id = dbus_g_proxy_begin_call_internal (proxy, method, notify,
+          user_data, destroy, arg_values, timeout);
 
-  g_value_array_free (arg_values);
+      g_value_array_free (arg_values);
+    }
 
   va_end (args);
 
@@ -2600,7 +2722,10 @@ dbus_g_proxy_begin_call_with_timeout (DBusGProxy          *proxy,
  * @proxy: a proxy for a remote interface
  * @call: the pending call ID from dbus_g_proxy_begin_call()
  * @error: return location for an error
- * @first_arg_type: type of first "out" argument
+ * @first_arg_type: type of first "out" argument, or %G_TYPE_INVALID if
+ *    there are no "out" arguments
+ * @...: return location for first "out" argument, followed by any further
+ *    type/location pairs, followed by %G_TYPE_INVALID
  *
  * Collects the results of a method call. The method call was normally
  * initiated with dbus_g_proxy_end_call(). You may use this function
@@ -2608,14 +2733,17 @@ dbus_g_proxy_begin_call_with_timeout (DBusGProxy          *proxy,
  * case this function will block if the results haven't yet been
  * received.
  *
- * If the call results in an error, the error is set as normal for
- * GError and the function returns #FALSE.
+ * All D-Bus method calls can fail with a remote error. If this occurs,
+ * the @error will be set and this function will return %FALSE.
  *
  * Otherwise, the "out" parameters and return value of the
  * method are stored in the provided varargs list.
  * The list should be terminated with G_TYPE_INVALID.
  *
- * Returns: #FALSE if an error is set.
+ * Returns: %TRUE on success
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is g_dbus_proxy_call_finish().
  */
 gboolean
 dbus_g_proxy_end_call (DBusGProxy          *proxy,
@@ -2643,7 +2771,10 @@ dbus_g_proxy_end_call (DBusGProxy          *proxy,
  * @proxy: a proxy for a remote interface
  * @method: method to invoke
  * @error: return location for an error
- * @first_arg_type: type of first "in" argument
+ * @first_arg_type: type of first "in" argument, or %G_TYPE_INVALID if none
+ * @...: value of first "in" argument, any further type/value pairs,
+ *    %G_TYPE_INVALID, type/location pairs for "out" arguments,
+ *    and %G_TYPE_INVALID again
  *
  * Function for synchronously invoking a method and receiving reply
  * values.  This function is equivalent to dbus_g_proxy_begin_call
@@ -2655,7 +2786,10 @@ dbus_g_proxy_end_call (DBusGProxy          *proxy,
  * It is an error to call this method on a proxy that has emitted
  * the #DBusGProxy::destroy signal.
  *
- * Returns: #FALSE if an error is set, #TRUE otherwise.
+ * Returns: %TRUE if the method succeeds, %FALSE if it fails
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is g_dbus_proxy_call_sync().
  */
 gboolean
 dbus_g_proxy_call (DBusGProxy        *proxy,
@@ -2665,7 +2799,7 @@ dbus_g_proxy_call (DBusGProxy        *proxy,
 		   ...)
 {
   gboolean ret;
-  guint call_id;
+  guint call_id = 0;
   va_list args;
   GValueArray *in_args;
   DBusGProxyPrivate *priv;
@@ -2679,23 +2813,18 @@ dbus_g_proxy_call (DBusGProxy        *proxy,
 
   DBUS_G_VALUE_ARRAY_COLLECT_ALL (in_args, first_arg_type, args);
 
-  call_id = dbus_g_proxy_begin_call_internal (proxy, method, NULL, NULL, NULL, in_args, priv->default_timeout);
-
-  g_value_array_free (in_args);
-
-  if (call_id > 0)
+  if (in_args != NULL)
     {
-      first_arg_type = va_arg (args, GType);
-      ret = dbus_g_proxy_end_call_internal (proxy, call_id, error, first_arg_type, args);
-    }
-  else
-    {
-      g_set_error (error, DBUS_GERROR,
-		   DBUS_GERROR_FAILED,
-		   "Disconnection or out-of-memory");
-      ret = FALSE;
+      call_id = dbus_g_proxy_begin_call_internal (proxy, method, NULL, NULL,
+          NULL, in_args, priv->default_timeout);
+
+      g_value_array_free (in_args);
     }
 
+  first_arg_type = va_arg (args, GType);
+  ret = dbus_g_proxy_end_call_internal (proxy, call_id, error, first_arg_type,
+      args);
+
   va_end (args);
 
   return ret;
@@ -2708,6 +2837,7 @@ dbus_g_proxy_call (DBusGProxy        *proxy,
  * @timeout: the timeout in milliseconds, or -1 to use a default
  * @error: return location for an error
  * @first_arg_type: type of first "in" argument
+ * @...: as for dbus_g_proxy_call()
  *
  * Function for synchronously invoking a method and receiving reply
  * values.  This function is equivalent to dbus_g_proxy_begin_call
@@ -2719,7 +2849,10 @@ dbus_g_proxy_call (DBusGProxy        *proxy,
  * It is an error to call this method on a proxy that has emitted
  * the #DBusGProxy::destroy signal.
  *
- * Returns: #FALSE if an error is set, #TRUE otherwise.
+ * Returns: %TRUE if the method succeeds, %FALSE if it fails
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is g_dbus_proxy_call_sync().
  */
 gboolean
 dbus_g_proxy_call_with_timeout (DBusGProxy        *proxy,
@@ -2730,7 +2863,7 @@ dbus_g_proxy_call_with_timeout (DBusGProxy        *proxy,
                    ...)
 {
   gboolean ret;
-  guint call_id;
+  guint call_id = 0;
   va_list args;
   GValueArray *in_args;
 
@@ -2743,12 +2876,17 @@ dbus_g_proxy_call_with_timeout (DBusGProxy        *proxy,
 
   DBUS_G_VALUE_ARRAY_COLLECT_ALL (in_args, first_arg_type, args);
 
-  call_id = dbus_g_proxy_begin_call_internal (proxy, method, NULL, NULL, NULL, in_args,timeout);
+  if (in_args != NULL)
+    {
+      call_id = dbus_g_proxy_begin_call_internal (proxy, method, NULL, NULL,
+          NULL, in_args, timeout);
 
-  g_value_array_free (in_args);
+      g_value_array_free (in_args);
+    }
 
   first_arg_type = va_arg (args, GType);
-  ret = dbus_g_proxy_end_call_internal (proxy, call_id, error, first_arg_type, args);
+  ret = dbus_g_proxy_end_call_internal (proxy, call_id, error,
+      first_arg_type, args);
 
   va_end (args);
 
@@ -2759,7 +2897,10 @@ dbus_g_proxy_call_with_timeout (DBusGProxy        *proxy,
  * dbus_g_proxy_call_no_reply:
  * @proxy: a proxy for a remote interface
  * @method: the name of the method to invoke
- * @first_arg_type: type of the first argument
+ * @first_arg_type: type of the first argument, or %G_TYPE_INVALID to call
+ *    the method without arguments
+ * @...: the first argument and any remaining type/argument pairs, followed by
+ *    %G_TYPE_INVALID to terminate the list
  *
  * Sends a method call message as with dbus_g_proxy_begin_call(), but
  * does not ask for a reply or allow you to receive one.
@@ -2769,6 +2910,9 @@ dbus_g_proxy_call_with_timeout (DBusGProxy        *proxy,
  *
  * TODO: this particular function shouldn't die on out of memory,
  * since you should be able to do a call with large arguments.
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is g_dbus_proxy_call() with @callback = %NULL.
  */
 void
 dbus_g_proxy_call_no_reply (DBusGProxy               *proxy,
@@ -2776,7 +2920,7 @@ dbus_g_proxy_call_no_reply (DBusGProxy               *proxy,
 			    GType                     first_arg_type,
 			    ...)
 {
-  DBusMessage *message;
+  DBusMessage *message = NULL;
   va_list args;
   GValueArray *in_args;
   DBusGProxyPrivate *priv;
@@ -2790,9 +2934,13 @@ dbus_g_proxy_call_no_reply (DBusGProxy               *proxy,
   va_start (args, first_arg_type);
   DBUS_G_VALUE_ARRAY_COLLECT_ALL (in_args, first_arg_type, args);
 
-  message = dbus_g_proxy_marshal_args_to_message (proxy, method, in_args);
+  if (in_args != NULL)
+    {
+      message = dbus_g_proxy_marshal_args_to_message (proxy, method, in_args);
+
+      g_value_array_free (in_args);
+    }
 
-  g_value_array_free (in_args);
   va_end (args);
 
   /* can only happen on a programming error or OOM; we already critical'd */
@@ -2821,6 +2969,9 @@ dbus_g_proxy_call_no_reply (DBusGProxy               *proxy,
  *
  * It is an error to call this method on a proxy that has emitted
  * the #DBusGProxy::destroy signal.
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is g_cancellable_cancel().
  */
 void
 dbus_g_proxy_cancel_call (DBusGProxy        *proxy,
@@ -2837,6 +2988,12 @@ dbus_g_proxy_cancel_call (DBusGProxy        *proxy,
 
   call_id = DBUS_G_PROXY_CALL_TO_ID (call);
 
+  if (call_id == 0)
+    {
+      /* nothing to cancel */
+      return;
+    }
+
   pending = g_hash_table_lookup (priv->pending_calls, GUINT_TO_POINTER (call_id));
   g_hash_table_remove (priv->pending_calls, GUINT_TO_POINTER (call_id));
   g_return_if_fail (pending != NULL);
@@ -2848,7 +3005,7 @@ dbus_g_proxy_cancel_call (DBusGProxy        *proxy,
  * dbus_g_proxy_send:
  * @proxy: a proxy for a remote interface
  * @message: the message to address and send
- * @client_serial: return location for message's serial, or #NULL 
+ * @client_serial: return location for message's serial, or %NULL
  *
  * Sends a message to the interface we're proxying for.  Does not
  * block or wait for a reply. The message is only actually written out
@@ -2866,6 +3023,9 @@ dbus_g_proxy_cancel_call (DBusGProxy        *proxy,
  *
  * It is an error to call this method on a proxy that has emitted
  * the #DBusGProxy::destroy signal.
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is g_dbus_connection_send_message().
  */
 void
 dbus_g_proxy_send (DBusGProxy          *proxy,
@@ -2909,7 +3069,8 @@ array_free_all (gpointer array)
  * dbus_g_proxy_add_signal:
  * @proxy: the proxy for a remote interface
  * @signal_name: the name of the signal
- * @first_type: the first argument type, or G_TYPE_INVALID if none
+ * @first_type: the first argument type, or %G_TYPE_INVALID if none
+ * @...: any subsequent argument types, followed by %G_TYPE_INVALID
  *
  * Specifies the argument signature of a D-Bus signal. When the signal is
  * emitted by the remote object, if the GTypes corresponding to its arguments'
@@ -2920,6 +3081,8 @@ array_free_all (gpointer array)
  *
  * It is also an error to call this method on a proxy that has emitted
  * the #DBusGProxy::destroy signal.
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 void
 dbus_g_proxy_add_signal  (DBusGProxy        *proxy,
@@ -2957,12 +3120,6 @@ dbus_g_proxy_add_signal  (DBusGProxy        *proxy,
     }
   va_end (args);
 
-#ifndef G_DISABLE_CHECKS
-  if (_dbus_gobject_lookup_marshaller (G_TYPE_NONE, gtypesig->len, (const GType*) gtypesig->data) == NULL)
-    g_warning ("No marshaller for signature of signal '%s'", signal_name);
-#endif
-
-  
   g_datalist_id_set_data_full (&priv->signal_signatures,
                                q, gtypesig,
                                array_free_all);
@@ -2984,6 +3141,9 @@ dbus_g_proxy_add_signal  (DBusGProxy        *proxy,
  *
  * It is an error to call this method on a proxy that has emitted
  * the #DBusGProxy::destroy signal.
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is g_dbus_connection_signal_subscribe().
  */
 void
 dbus_g_proxy_connect_signal (DBusGProxy             *proxy,
@@ -3038,6 +3198,9 @@ dbus_g_proxy_connect_signal (DBusGProxy             *proxy,
  *
  * It is an error to call this method on a proxy that has emitted
  * the #DBusGProxy::destroy signal.
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is g_dbus_connection_signal_unsubscribe().
  */
 void
 dbus_g_proxy_disconnect_signal (DBusGProxy             *proxy,
@@ -3100,6 +3263,8 @@ dbus_g_proxy_disconnect_signal (DBusGProxy             *proxy,
  * the #DBusGProxy::destroy signal.
  *
  * Since: 0.75
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 void
 dbus_g_proxy_set_default_timeout (DBusGProxy        *proxy,
