@@ -4,6 +4,8 @@
  * Copyright (C) 2005 Red Hat, Inc.
  * Copyright (C) 2007 Codethink Ltd.
  *
+ * SPDX-License-Identifier: AFL-2.1 OR GPL-2.0-or-later
+ *
  * Licensed under the Academic Free License version 2.1
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -33,23 +35,35 @@
 
 /**
  * SECTION:dbus-gtype-specialized
+ * @title: Specializable GType System
  * @short_description: Specialized GTypes
  * @stability: Unstable
  *
  * Specialized gtypes are basically a way to allow the definition of
- * recursive GTypes. It allows the definition of 'containers' which is
- * basically a user defined structure capabale of holding other data and a 
+ * recursive #GTypes. It allows the definition of 'containers' which is
+ * basically a user defined structure capable of holding other data, and a
  * set of functions defining how to access that structure. Containers come in
- * 3 flavors: collections, maps and structs. 
+ * 3 flavors: collections, maps and structs.
  *
  * A collection is a container that holds an ordered set of items, all
- * of which must be the same type. 
+ * of which must be the same type. (This is an <emphasis>array</emphasis>
+ * in standard D-Bus terminology.) dbus-glib specialized collections can be
+ * #GArray (for numeric elements), #GPtrArray (for string, object or
+ * boxed elements), #GSList (for boxed elements, not recommended), or a
+ * user-defined type.
  *
- * A map is a container that holds a set of key/value pairs. 
- * The keys have one type, and the values another.
+ * A map is a container that holds a set of key/value pairs.
+ * The keys have one type, and the values another; the type of the keys
+ * must be a numeric or string-like type. (This is a <emphasis>dict</emphasis>
+ * (dictionary) or <emphasis>array of dict entry</emphasis> in standard D-Bus
+ * terminology.) dbus-glib specialized maps can be #GHashTable or a
+ * user-defined type.
  *
- * A struct is a container that holds a fixed number of members, each member 
- * having a predefined type.
+ * A struct is a container that holds a fixed number of members, each member
+ * having a predefined type. (This is a <emphasis>struct</emphasis> in
+ * standard D-Bus terminology.) It is analogous to the C @struct keyword, but
+ * dbus-glib does not generally represent D-Bus structs in C structs.
+ * dbus-glib specialized structs can be #GValueArray or a user-defined type.
  *
  * A specialization is a GType detailing a particular container with
  * particular types (a type specialization).
@@ -59,6 +73,329 @@
  * This documentation needs splitting into two pages, one for defining new
  * containers and using existing containers. I expect most users to only do
  * the latter. I also need to add some examples.
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is #GVariant, which is much better-designed.
+ */
+
+/**
+ * DBUS_TYPE_G_BOOLEAN_ARRAY:
+ *
+ * Expands to a function call returning the #GType of a #GArray of #gboolean
+ * (corresponding to the D-Bus signature "ab").
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is `G_VARIANT_TYPE ("ab")`.
+ */
+
+/**
+ * DBUS_TYPE_G_INT_ARRAY:
+ *
+ * Expands to a function call returning the #GType of a #GArray of #gint
+ * (corresponding to the D-Bus signature "ai").
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is `G_VARIANT_TYPE ("ai")`.
+ */
+
+/**
+ * DBUS_TYPE_G_UINT_ARRAY:
+ *
+ * Expands to a function call returning the #GType of a #GArray of #guint
+ * (corresponding to the D-Bus signature "au").
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is `G_VARIANT_TYPE ("au")`.
+ */
+
+/**
+ * DBUS_TYPE_G_INT64_ARRAY:
+ *
+ * Expands to a function call returning the #GType of a #GArray of #gint64
+ * (corresponding to the D-Bus signature "ax").
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is `G_VARIANT_TYPE ("ax")`.
+ */
+
+/**
+ * DBUS_TYPE_G_UINT64_ARRAY:
+ *
+ * Expands to a function call returning the #GType of a #GArray of #guint64
+ * (corresponding to the D-Bus signature "at").
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is `G_VARIANT_TYPE ("at")`.
+ */
+
+/**
+ * DBUS_TYPE_G_UCHAR_ARRAY:
+ *
+ * Expands to a function call returning the #GType of a #GArray of #guchar
+ * (corresponding to the D-Bus signature "ay").
+ *
+ * Note that this is not the same thing as a #GByteArray! dbus-glib does not
+ * know about the #GByteArray type.
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is %G_VARIANT_TYPE_BYTESTRING.
+ */
+
+/**
+ * DBUS_TYPE_G_OBJECT_ARRAY:
+ *
+ * Expands to a function call returning the #GType of a #GPtrArray of #GObject.
+ *
+ * Use this type with caution: it can sometimes be used as a representation
+ * of arrays whose D-Bus signature is "ao" (transferred as an array of object
+ * paths), but the conventional type for such arrays is
+ * <literal>(dbus_g_type_get_collection ("GPtrArray",
+ * DBUS_TYPE_G_OBJECT_PATH))</literal>.
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is %G_VARIANT_TYPE_OBJECT_PATH_ARRAY.
+ */
+
+/**
+ * DBUS_TYPE_G_STRING_STRING_HASHTABLE:
+ *
+ * Expands to a function call returning the #GType of a #GHashTable where the
+ * keys are strings and the values are also strings (corresponding to the
+ * D-Bus signature "a{ss}").
+ *
+ * Deprecated: New code should use GDBus instead. The closest equivalent
+ *  is `G_VARIANT_TYPE ("a{ss}")`.
+ */
+
+/**
+ * DBusGTypeSpecializedVtable:
+ * @constructor: returns a new, blank instance of the @type
+ * @free_func: if not %NULL, frees the @type instance @val
+ * @copy_func: returns a "deep copy" of the @type instance @val
+ * @simple_free_func: if not %NULL, frees its argument
+ *
+ * A table of methods used to implement specialized container behaviour on
+ * user-defined collections, maps and structs. Exactly one of @free_func and
+ * @simple_free_func must be implemented; the other must be %NULL.
+ * @constructor and @copy_func must always be implemented.
+ *
+ * There are additional members, which are reserved for future expansion and
+ * must be %NULL.
+ *
+ * Deprecated: New code should use GDBus instead.
+ */
+
+/**
+ * DBusGTypeSpecializedConstructor:
+ * @type: a specialized boxed type
+ *
+ * <!-- -->
+ *
+ * Returns: a new instance of @type
+ *
+ * Deprecated: New code should use GDBus instead.
+ */
+
+/**
+ * DBusGTypeSpecializedFreeFunc:
+ * @type: a specialized boxed type
+ * @val: an instance of @type
+ *
+ * Frees @val according to @type. This is analogous to #GBoxedFreeFunc, but
+ * can use information from @type (for instance to free the contents of a
+ * container before freeing the actual container).
+ *
+ * Deprecated: New code should use GDBus instead.
+ */
+
+/**
+ * DBusGTypeSpecializedCopyFunc:
+ * @type: a specialized boxed type
+ * @src: an instance of @type
+ *
+ * Copies @src according to @type. This is analogous to #GBoxedCopyFunc, but
+ * can use information from @type (for instance to copy each element of a
+ * collection).
+ *
+ * Returns: a deep copy of @src
+ *
+ * Deprecated: New code should use GDBus instead.
+ */
+
+/**
+ * DBusGTypeSpecializedCollectionFixedAccessorFunc:
+ * @type: a specialized collection boxed type
+ * @instance: an instance of @type
+ * @values: used to return a pointer to the contents of @instance
+ * @len: used to return the number of elements in @instance
+ *
+ * Implements dbus_g_type_collection_get_fixed() for a #GValue with type
+ * @type, containing @instance.
+ *
+ * Returns: %TRUE on success
+ *
+ * Deprecated: New code should use GDBus instead.
+ */
+
+/**
+ * DBusGTypeSpecializedCollectionIterator:
+ * @value: an element of the collection
+ * @user_data: the data supplied when calling
+ *    dbus_g_type_collection_value_iterate()
+ *
+ * A library-user-supplied function, called for each element in the
+ * collection when dbus_g_type_collection_value_iterate() is called.
+ *
+ * Deprecated: New code should use GDBus instead.
+ */
+
+/**
+ * DBusGTypeSpecializedCollectionIteratorFunc:
+ * @type: a specialized collection boxed type
+ * @instance: an instance of @type
+ * @iterator: the function to call for each element
+ * @user_data: data to pass to @iterator
+ *
+ * Implements dbus_g_type_collection_value_iterate() for a #GValue with
+ * type @type, containing @instance.
+ *
+ * Deprecated: New code should use GDBus instead.
+ */
+
+/**
+ * DBusGTypeSpecializedCollectionAppendFunc:
+ * @ctx: an appending context returned by dbus_g_type_specialized_init_append()
+ * @val: a value to copy into the collection
+ *
+ * Implements dbus_g_type_specialized_collection_append().
+ *
+ * This function should use the @val and @specialization_type members of @ctx.
+ *
+ * Deprecated: New code should use GDBus instead.
+ */
+
+/**
+ * DBusGTypeSpecializedCollectionEndAppendFunc:
+ * @ctx: an appending context returned by dbus_g_type_specialized_init_append()
+ *
+ * Implements dbus_g_type_specialized_collection_end_append().
+ *
+ * This function should use the @val and @specialization_type members of @ctx.
+ *
+ * Deprecated: New code should use GDBus instead.
+ */
+
+/**
+ * DBusGTypeSpecializedCollectionVtable:
+ * @base_vtable: base methods shared between collections and other types
+ * @fixed_accessor: if not %NULL, provides access to the contents of this
+ *    collection, as documented for dbus_g_type_collection_get_fixed()
+ * @iterator: iterates through the members of @instance
+ * @append_func: appends a new member to @instance
+ * @end_append_func: if not %NULL, called after each group of calls to
+ *    the @append_func
+ *
+ * A table of methods used to implement specialized collection behaviour
+ * on user-defined types. At least @iterator and @append_func must be
+ * implemented.
+ *
+ * Deprecated: New code should use GDBus instead.
+ */
+
+/**
+ * DBusGTypeSpecializedMapIterator:
+ * @key_val: a key from the map
+ * @value_val: a value from the map
+ * @user_data: the data supplied when calling
+ *    dbus_g_type_map_value_iterate()
+ *
+ * A library-user-supplied function, called for each key/value pair in the
+ * collection when dbus_g_type_map_value_iterate() is called.
+ *
+ * Deprecated: New code should use GDBus instead.
+ */
+
+/**
+ * DBusGTypeSpecializedMapIteratorFunc:
+ * @type: a specialized map boxed type
+ * @instance: an instance of @type
+ * @iterator: the function to call for each key/value pair
+ * @user_data: data to pass to @iterator
+ *
+ * Implements dbus_g_type_map_value_iterate() for a #GValue with
+ * type @type, containing @instance.
+ *
+ * Deprecated: New code should use GDBus instead.
+ */
+
+/**
+ * DBusGTypeSpecializedMapAppendFunc:
+ * @ctx: an appending context returned by dbus_g_type_specialized_init_append()
+ * @key: a key to add to the collection
+ * @val: a value to add to the collection
+ *
+ * Implements dbus_g_type_specialized_map_append().
+ *
+ * This function should use the @val and @specialization_type members of @ctx,
+ * and replace any existing value with key equal to @key.
+ *
+ * Deprecated: New code should use GDBus instead.
+ */
+
+/**
+ * DBusGTypeSpecializedMapVtable:
+ * @base_vtable: base methods shared between maps and other types
+ * @iterator: iterates through the members of @instance
+ * @append_func: adds a new key/value pair to @instance
+ *
+ * A table of methods used to implement specialized collection behaviour
+ * on user-defined types. Both methods must be implemented.
+ *
+ * Deprecated: New code should use GDBus instead.
+ */
+
+/**
+ * DBusGTypeSpecializedStructGetMember:
+ * @type: a specialized struct boxed type
+ * @instance: an instance of @type
+ * @member: the index of the member, starting from 0
+ * @ret_value: an initialized #GValue of the appropriate type for the given
+ *    member of @type
+ *
+ * Implements dbus_g_type_struct_get_member() for a #GValue with type @type,
+ * containing @instance.
+ *
+ * Returns: %TRUE on success
+ *
+ * Deprecated: New code should use GDBus instead.
+ */
+
+/**
+ * DBusGTypeSpecializedStructSetMember:
+ * @type: a specialized struct boxed type
+ * @instance: an instance of @type
+ * @member: the index of the member, starting from 0
+ * @new_value: an initialized #GValue of the appropriate type for the given
+ *    member of @type
+ *
+ * Implements dbus_g_type_struct_set_member() for a #GValue with type @type,
+ * containing @instance.
+ *
+ * Returns: %TRUE on success
+ *
+ * Deprecated: New code should use GDBus instead.
+ */
+
+/**
+ * DBusGTypeSpecializedStructVtable:
+ * @base_vtable: base methods shared between maps and other types
+ * @get_member: returns a member by its index
+ * @set_member: sets a member by its index
+ *
+ * A table of methods used to implement specialized collection behaviour
+ * on user-defined types. Both methods must be implemented.
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 
 typedef enum {
@@ -102,6 +439,17 @@ specialized_init (gpointer arg G_GNUC_UNUSED)
   return NULL;
 }
 
+/**
+ * dbus_g_type_specialized_init:
+ *
+ * Initialize dbus-glib specialized #GType<!-- -->s.
+ *
+ * In older versions of dbus-glib, it was necessary to do this before
+ * instantiating or registering any specialized type. It is now done
+ * automatically whenever necessary.
+ *
+ * Deprecated: New code should use GDBus instead.
+ */
 void
 dbus_g_type_specialized_init (void)
 {
@@ -260,7 +608,13 @@ register_container (const char                         *name,
 		    const DBusGTypeSpecializedVtable   *vtable)
 {
   DBusGTypeSpecializedContainer *klass;
-  
+
+  g_warn_if_fail (vtable->constructor != NULL);
+  /* must have either free_func or simple_free_func */
+  g_warn_if_fail (vtable->free_func != NULL ||
+                  vtable->simple_free_func != NULL);
+  g_warn_if_fail (vtable->copy_func != NULL);
+
   klass = g_new0 (DBusGTypeSpecializedContainer, 1);
   klass->type = type;
   klass->vtable = vtable;
@@ -275,6 +629,8 @@ register_container (const char                         *name,
  * @flags: As yet unused.
  *
  * Defines a new collection container.
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 void
 dbus_g_type_register_collection (const char                                   *name,
@@ -291,6 +647,11 @@ _dbus_g_type_register_collection (const char                                   *
 				  const DBusGTypeSpecializedCollectionVtable   *vtable,
 				  guint                                         flags)
 {
+  /* fixed_accessor is optional */
+  g_warn_if_fail (vtable->iterator != NULL);
+  g_warn_if_fail (vtable->append_func != NULL);
+  /* end_append_func is optional */
+
   register_container (name, DBUS_G_SPECTYPE_COLLECTION, (const DBusGTypeSpecializedVtable*) vtable);
 }
 
@@ -301,6 +662,8 @@ _dbus_g_type_register_collection (const char                                   *
  * @flags: As yet unused.
  *
  * Defines a new map container.
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 void
 dbus_g_type_register_map (const char                            *name,
@@ -317,6 +680,9 @@ _dbus_g_type_register_map (const char                            *name,
 			   const DBusGTypeSpecializedMapVtable   *vtable,
 			   guint                                  flags)
 {
+  g_warn_if_fail (vtable->iterator != NULL);
+  g_warn_if_fail (vtable->append_func != NULL);
+
   register_container (name, DBUS_G_SPECTYPE_MAP, (const DBusGTypeSpecializedVtable*) vtable);
 }
 
@@ -327,6 +693,8 @@ _dbus_g_type_register_map (const char                            *name,
  * @flags: As yet unused.
  *
  * Defines a new struct container.
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 void
 dbus_g_type_register_struct (const char                             *name,
@@ -343,6 +711,9 @@ _dbus_g_type_register_struct (const char                             *name,
 			      const DBusGTypeSpecializedStructVtable *vtable,
 			      guint                                   flags)
 {
+  g_warn_if_fail (vtable->get_member != NULL);
+  g_warn_if_fail (vtable->set_member != NULL);
+
   register_container (name, DBUS_G_SPECTYPE_STRUCT, (const DBusGTypeSpecializedVtable*) vtable);
 }
 
@@ -351,6 +722,10 @@ _dbus_g_type_register_struct (const char                             *name,
  * @map_type: a gtype of a map specialization
  *
  * Peek the vtable for a given map specialization
+ *
+ * Returns: the vtable
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 const DBusGTypeSpecializedMapVtable* dbus_g_type_map_peek_vtable (GType map_type)
 {
@@ -368,6 +743,10 @@ const DBusGTypeSpecializedMapVtable* dbus_g_type_map_peek_vtable (GType map_type
  * @collection_type: a gtype of a collection specialization
  *
  * Peek the vtable for a given collection specialization
+ *
+ * Returns: the vtable
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 const DBusGTypeSpecializedCollectionVtable* dbus_g_type_collection_peek_vtable (GType collection_type)
 {
@@ -382,9 +761,13 @@ const DBusGTypeSpecializedCollectionVtable* dbus_g_type_collection_peek_vtable (
 
 /**
  * dbus_g_type_struct_peek_vtable:
- * @collection_type: a gtype of a struct specialization
+ * @struct_type: a gtype of a struct specialization
  *
  * Peek the vtable for a given struct specialization
+ *
+ * Returns: the vtable
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 const DBusGTypeSpecializedStructVtable* dbus_g_type_struct_peek_vtable (GType struct_type)
 {
@@ -482,6 +865,8 @@ lookup_or_register_specialized (const char  *container,
  * creating the type if not already created.
  *
  * Returns: the #GType of that instance
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 GType
 dbus_g_type_get_collection (const char *container,
@@ -500,6 +885,8 @@ dbus_g_type_get_collection (const char *container,
  * creating the type if not already created.
  *
  * Returns: the #GType of that instance
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 GType
 dbus_g_type_get_map (const char   *container,
@@ -522,6 +909,8 @@ dbus_g_type_get_map (const char   *container,
  * creating the type if not already created.
  *
  * Returns: the #GType of that instance
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 GType
 dbus_g_type_get_structv (const char   *container,
@@ -537,10 +926,12 @@ dbus_g_type_get_structv (const char   *container,
  * @first_type: #GType for the struct's first member
  * @...: more GTypes for the struct's members, terminated by G_TYPE_INVALID
  *
- * Varags methsod to get a #GType for a particular struct instance,
+ * Varags method to get a #GType for a particular struct instance,
  * creating the type if not already created.
  *
  * Returns: the #GType of that instance
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 GType
 dbus_g_type_get_struct (const char *container,
@@ -577,6 +968,8 @@ dbus_g_type_get_struct (const char *container,
  * Tests if a given GType is a collection.
  *
  * Returns: true if the given GType is a collection
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 gboolean
 dbus_g_type_is_collection (GType gtype)
@@ -593,9 +986,11 @@ dbus_g_type_is_collection (GType gtype)
  * @gtype: a GType to test
  *
  * Tests if a given GType is a map,
- * i.e. it was created with #dbus_g_type_get_map.
+ * i.e. it was created with dbus_g_type_get_map().
  *
  * Returns: true if the given GType is a map
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 gboolean
 dbus_g_type_is_map (GType gtype)
@@ -612,9 +1007,11 @@ dbus_g_type_is_map (GType gtype)
  * @gtype: a GType to test
  *
  * Tests if a given GType is a struct,
- * i.e. it was created with #dbus_g_type_get_struct
+ * i.e. it was created with dbus_g_type_get_struct()
  *
  * Returns: true if the given GType is a struct
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 gboolean
 dbus_g_type_is_struct (GType gtype)
@@ -641,10 +1038,14 @@ get_specialization_index (GType gtype, guint i)
 
 /**
  * dbus_g_type_get_collection_specialization:
- * @gtype: a collection GType, as created by #dbus_g_type_get_collection.
+ * @gtype: a collection #GType, as created by dbus_g_type_get_collection()
+ *
+ * Return the type of each element in collections of type @gtype.
+ * It is an error to call this function on a non-collection type.
  *
  * Returns: the element type for a given collection GType.
- * Returns G_TYPE_INVALID if not a collection GType
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 GType
 dbus_g_type_get_collection_specialization (GType gtype)
@@ -655,10 +1056,14 @@ dbus_g_type_get_collection_specialization (GType gtype)
 
 /**
  * dbus_g_type_get_map_key_specialization:
- * @gtype: a map GType, as created by #dbus_g_type_get_map.
+ * @gtype: a map #GType, as created by dbus_g_type_get_map()
  *
- * Returns: the key type for a given map GType.
- * Returns G_TYPE_INVALID if not a map GType
+ * Return the type of the keys in maps of type @gtype.
+ * It is an error to call this function on a non-map type.
+ *
+ * Returns: the key type for a given map #GType.
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 GType
 dbus_g_type_get_map_key_specialization (GType gtype)
@@ -669,10 +1074,14 @@ dbus_g_type_get_map_key_specialization (GType gtype)
 
 /**
  * dbus_g_type_get_map_value_specialization:
- * @gtype: a map GType, as created by #dbus_g_type_get_map.
+ * @gtype: a map GType, as created by dbus_g_type_get_map().
+ *
+ * Return the type of the values in maps of type @gtype.
+ * It is an error to call this function on a non-map type.
  *
  * Returns: the value type for a given map GType.
- * Returns G_TYPE_INVALID if not a map GType
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 GType
 dbus_g_type_get_map_value_specialization (GType gtype)
@@ -683,11 +1092,16 @@ dbus_g_type_get_map_value_specialization (GType gtype)
 
 /**
  * dbus_g_type_get_struct_member_type
- * @gtype: a struct GType, as created with #dbus_g_type_get_struct
+ * @gtype: a struct GType, as created with dbus_g_type_get_struct()
  * @member: the index of a struct member
  *
- * Returns: the type for a given member of a struct #GType.
- * Returns G_TYPE_INVALID if not a struct GType
+ * Get the type of a member of a specialized struct.
+ * It is an error to call this function on a non-struct type.
+ *
+ * Returns: the type for a given member of a struct #GType,
+ *    or %G_TYPE_INVALID if @member &gt;= dbus_g_type_get_struct_size()
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 GType
 dbus_g_type_get_struct_member_type (GType gtype, guint member)
@@ -698,10 +1112,14 @@ dbus_g_type_get_struct_member_type (GType gtype, guint member)
 
 /**
  * dbus_g_type_get_struct_size
- * @gtype: a struct GType, as created with #dbus_g_type_get_struct.
+ * @gtype: a struct GType, as created with dbus_g_type_get_struct().
+ *
+ * Get the number of members in a specialized struct.
+ * It is an error to call this function on a non-struct type.
  *
  * Returns: the number of members in a given struct #GType.
- * Returns G_TYPE_INVALID if not a struct GType
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 guint
 dbus_g_type_get_struct_size (GType gtype)
@@ -715,7 +1133,8 @@ dbus_g_type_get_struct_size (GType gtype)
 
 /**
  * dbus_g_type_specialized_construct:
- * @gtype: a specialized #GType, as created by #dbus_g_type_get_collection, #dbus_g_type_get_map or #dbus_g_type_get_struct.
+ * @gtype: a specialized #GType, as created by dbus_g_type_get_collection(),
+ *  dbus_g_type_get_map() or dbus_g_type_get_struct()
  *
  * Create an instance of a given specialized type. 
  * The structure created and returned will depend on the container type of the 
@@ -724,6 +1143,8 @@ dbus_g_type_get_struct_size (GType gtype)
  * then this will return a GArray with element_size of sizeof(int)
  *
  * Returns: a pointer to a newly constructed instance of the given type.
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 gpointer
 dbus_g_type_specialized_construct (GType gtype)
@@ -740,11 +1161,31 @@ dbus_g_type_specialized_construct (GType gtype)
 
 /**
  * dbus_g_type_collection_get_fixed:
- * @deprecated: maybe?
+ * @value: a GValue containing a boxed specialized collection
+ *    that has a @fixed_accessor in its vtable
+ * @data_ret: used to return a pointer to the fixed data, which must not be
+ *    modified (for instance, for a #GArray of #gint, this would point
+ *    to an array of #gint)
+ * @len_ret: used to return the length (counting collection elements, not
+ *    bytes: in a #GArray containing one #gint, this would be 1)
  *
- * if the collection has elements of fixed size (i.e. a fundamental type), 
- * return the contents of the array.
- * Its pretty obscure and I don't think anyone uses it.
+ * Calling this function is likely to be a bad idea. Consider using
+ * dbus_g_type_collection_value_iterate() instead.
+ *
+ * On success, @data_ret is a pointer to the underlying data in a collection
+ * of fixed-length fundamental types. Knowledge of the underlying data model
+ * of the collection is needed in order to use @data_ret correctly.
+ *
+ * It is an error to call this function on a specialized type that is not a
+ * collection, or on a collection that does not have a @fixed_accessor in its
+ * #DBusGTypeSpecializedCollectionVtable.
+ *
+ * Specialized #GArray<!---->s are the only types provided by dbus-glib that
+ * can be used with this function; user-defined types might also work.
+ *
+ * Returns: %TRUE on success
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 gboolean
 dbus_g_type_collection_get_fixed (GValue   *value,
@@ -752,6 +1193,7 @@ dbus_g_type_collection_get_fixed (GValue   *value,
 				  guint    *len_ret)
 {
   DBusGTypeSpecializedData *data;
+  DBusGTypeSpecializedCollectionVtable *vtable;
   GType gtype;
 
   dbus_g_type_specialized_init();
@@ -759,12 +1201,17 @@ dbus_g_type_collection_get_fixed (GValue   *value,
   g_return_val_if_fail (G_VALUE_HOLDS_BOXED (value), FALSE);
 
   gtype = G_VALUE_TYPE (value);
-  data = lookup_specialization_data (gtype);
-  g_return_val_if_fail (data != NULL, FALSE);
+  g_return_val_if_fail (dbus_g_type_is_collection (gtype), FALSE);
 
-  return ((DBusGTypeSpecializedCollectionVtable *) (data->klass->vtable))->fixed_accessor (gtype,
-											   g_value_get_boxed (value),
-											   data_ret, len_ret);
+  data = lookup_specialization_data (gtype);
+  /* dbus_g_type_is_collection() already checked this */
+  g_assert (data != NULL);
+
+  vtable = (DBusGTypeSpecializedCollectionVtable *) (data->klass->vtable);
+  g_return_val_if_fail (vtable->fixed_accessor != NULL, FALSE);
+
+  return vtable->fixed_accessor (gtype, g_value_get_boxed (value),
+                                 data_ret, len_ret);
 }
 
 /**
@@ -777,6 +1224,8 @@ dbus_g_type_collection_get_fixed (GValue   *value,
  * The function is passed a #GValue containing the element and the given 
  * @user_data parameter. The collection may not be modified while iterating over 
  * it.
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 void
 dbus_g_type_collection_value_iterate (const GValue                           *value,
@@ -791,28 +1240,47 @@ dbus_g_type_collection_value_iterate (const GValue                           *va
   g_return_if_fail (G_VALUE_HOLDS_BOXED (value));
 
   gtype = G_VALUE_TYPE (value);
+  g_return_if_fail (dbus_g_type_is_collection (gtype));
+
   data = lookup_specialization_data (gtype);
-  g_return_if_fail (data != NULL);
+  /* dbus_g_type_is_collection() already checked this */
+  g_assert (data != NULL);
 
   ((DBusGTypeSpecializedCollectionVtable *) data->klass->vtable)->iterator (gtype,
 									    g_value_get_boxed (value),
 									    iterator, user_data);
 }
 
+/**
+ * DBusGTypeSpecializedAppendContext:
+ * @val: the #GValue containing the array to which you're appending
+ * @specialization_type: the #GType of the array's elements
+ *
+ * A context for appending. There are more fields, which are private.
+ *
+ * Deprecated: New code should use GDBus instead.
+ */
+
 typedef struct {
   GValue *val;
   GType specialization_type;
   DBusGTypeSpecializedData *specdata;
+  guint c;
+  gpointer d;
 } DBusGTypeSpecializedAppendContextReal;
 
+G_STATIC_ASSERT (sizeof (DBusGTypeSpecializedAppendContextReal) ==
+                 sizeof (DBusGTypeSpecializedAppendContext));
+
 /**
  * dbus_g_type_specialized_init_append:
  * @value: a #GValue containing an instance of specialized type
  * @ctx: a #DBusGTypeSpecializedAppendContext in which to return a new appending context.
- * @deprecated: maybe i'll deprecate this as its a bit wank.
  *
  * Create a new context for adding elements to a collection or key/value pairs 
  * to a map. You generally don't need or want to use this..
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 void
 dbus_g_type_specialized_init_append (GValue *value, DBusGTypeSpecializedAppendContext *ctx)
@@ -836,43 +1304,55 @@ dbus_g_type_specialized_init_append (GValue *value, DBusGTypeSpecializedAppendCo
 
 /**
  * dbus_g_type_specialized_collection_append:
- * @ctx: a context created by #dbus_g_type_specialized_init_append
- * @elt: a GValue containing an element to append to the collection.
- * @deprecated: maybe i'll deprecate this as its a bit wank.
+ * @ctx: a context created by dbus_g_type_specialized_init_append()
+ *    for a #GValue containing a collection
+ * @elt: a #GValue containing an element to append to the collection
  *
  * Appends a given element to the end of a collection.
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 void
 dbus_g_type_specialized_collection_append (DBusGTypeSpecializedAppendContext *ctx,
 					   GValue                            *elt)
 {
   DBusGTypeSpecializedAppendContextReal *realctx = (DBusGTypeSpecializedAppendContextReal *) ctx;
+
+  g_return_if_fail (dbus_g_type_is_collection (G_VALUE_TYPE (ctx->val)));
+
   ((DBusGTypeSpecializedCollectionVtable *) realctx->specdata->klass->vtable)->append_func (ctx, elt);
 }
 
 /**
  * dbus_g_type_specialized_collection_end_append:
- * @ctx: a context created by #dbus_g_type_specialized_init_append
- * @deprecated: maybe i'll deprecate this as its a bit wank.
+ * @ctx: a context created by dbus_g_type_specialized_init_append()
+ *    for a #GValue containing a collection
  *
  * Finish appending elements to a given collection
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 void
 dbus_g_type_specialized_collection_end_append (DBusGTypeSpecializedAppendContext *ctx)
 {
   DBusGTypeSpecializedAppendContextReal *realctx = (DBusGTypeSpecializedAppendContextReal *) ctx;
+
+  g_return_if_fail (dbus_g_type_is_collection (G_VALUE_TYPE (ctx->val)));
+
   if (((DBusGTypeSpecializedCollectionVtable *) realctx->specdata->klass->vtable)->end_append_func != NULL)
     ((DBusGTypeSpecializedCollectionVtable *) realctx->specdata->klass->vtable)->end_append_func (ctx);
 }
 
 /**
  * dbus_g_type_specialized_map_append:
- * @ctx: a context created by #dbus_g_type_specialized_init_append
+ * @ctx: a context created by dbus_g_type_specialized_init_append()
+ *    for a #GValue containing a map
  * @key: a GValue containing a key, whose contents will be stolen by @ctx
  * @val: a GValue containing a value, whose contents will be stolen by @ctx
- * @deprecated: maybe i'll deprecate this as its a bit wank.
  *
  * Inserts the given key/value pair into the map instance.
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 void
 dbus_g_type_specialized_map_append (DBusGTypeSpecializedAppendContext *ctx,
@@ -880,13 +1360,16 @@ dbus_g_type_specialized_map_append (DBusGTypeSpecializedAppendContext *ctx,
 				    GValue                            *val)
 {
   DBusGTypeSpecializedAppendContextReal *realctx = (DBusGTypeSpecializedAppendContextReal *) ctx;
+
+  g_return_if_fail (dbus_g_type_is_map (G_VALUE_TYPE (ctx->val)));
+
   ((DBusGTypeSpecializedMapVtable *) realctx->specdata->klass->vtable)->append_func (ctx, key, val);
 }
 
 
 /**
  * dbus_g_type_map_value_iterate:
- * @value: a #GValue holding a collection type.
+ * @value: a #GValue holding a specialized map
  * @iterator: a function to call for each element
  * @user_data: user data to pass to the @iterator
  *
@@ -894,6 +1377,8 @@ dbus_g_type_specialized_map_append (DBusGTypeSpecializedAppendContext *ctx,
  * The function is passed two GValues containing the key/value pair and the given 
  * @user_data parameter. The map may not be modified while iterating over 
  * it.
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 void
 dbus_g_type_map_value_iterate (const GValue                           *value,
@@ -908,8 +1393,11 @@ dbus_g_type_map_value_iterate (const GValue                           *value,
   g_return_if_fail (G_VALUE_HOLDS_BOXED (value));
 
   gtype = G_VALUE_TYPE (value);
+  g_return_if_fail (dbus_g_type_is_map (gtype));
+
   data = lookup_specialization_data (gtype);
-  g_return_if_fail (data != NULL);
+  /* already checked by dbus_g_type_is_map() */
+  g_assert (data != NULL);
 
   ((DBusGTypeSpecializedMapVtable *) data->klass->vtable)->iterator (gtype,
 								     g_value_get_boxed (value),
@@ -926,7 +1414,9 @@ dbus_g_type_map_value_iterate (const GValue                           *value,
  * was the correct type for that member, e.g. as returned by 
  * @dbus_g_type_get_struct_member_type
  *
- * Returns: TRUE if sucessful
+ * Returns: %TRUE if successful
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 gboolean
 dbus_g_type_struct_get_member (const GValue *value,
@@ -941,8 +1431,11 @@ dbus_g_type_struct_get_member (const GValue *value,
   g_return_val_if_fail (G_VALUE_HOLDS_BOXED (value), FALSE);
 
   gtype = G_VALUE_TYPE (value);
+  g_return_val_if_fail (dbus_g_type_is_struct (gtype), FALSE);
+
   data = lookup_specialization_data (gtype);
-  g_return_val_if_fail (data != NULL, FALSE);
+  /* already checked by dbus_g_type_is_struct() */
+  g_assert (data != NULL);
 
   return ((DBusGTypeSpecializedStructVtable *) (data->klass->vtable))->get_member(gtype,
 											   g_value_get_boxed (value),
@@ -955,10 +1448,12 @@ dbus_g_type_struct_get_member (const GValue *value,
  * @member: the index of a given member
  * @src: an #GValue containing the new value for that struct member
  *
- * Sets a given member of a struct to a new value. The type of @src must match 
- * the exiting type of @member member of the struct.
+ * Sets a given member of a struct to a new value. The type of @src must match
+ * the existing type of @member member of the struct.
  *
- * Returns: TRUE if sucessful
+ * Returns: %TRUE if successful
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 gboolean
 dbus_g_type_struct_set_member (GValue       *value,
@@ -973,8 +1468,11 @@ dbus_g_type_struct_set_member (GValue       *value,
   g_return_val_if_fail (G_VALUE_HOLDS_BOXED (value), FALSE);
 
   gtype = G_VALUE_TYPE (value);
+  g_return_val_if_fail (dbus_g_type_is_struct (gtype), FALSE);
+
   data = lookup_specialization_data (gtype);
-  g_return_val_if_fail (data != NULL, FALSE);
+  /* already checked by dbus_g_type_is_struct() */
+  g_assert (data != NULL);
 
   return ((DBusGTypeSpecializedStructVtable *) (data->klass->vtable))->set_member(gtype,
 											   g_value_get_boxed (value),
@@ -983,16 +1481,18 @@ dbus_g_type_struct_set_member (GValue       *value,
 
 /**
  * dbus_g_type_struct_get:
- * @value: a #GValue containing a #DBusGTypeStruct type
+ * @value: a #GValue containing a struct instance
  * @member: struct member to get
  * @...: location in which to return the value of this member,
- *       followed optionally by more member/return locations pairs, followed by
- *       by #G_MAXUINT
+ *       followed optionally by more member/return locations pairs, followed
+ *       by %G_MAXUINT
  *
  * Collects the selected values of this struct into the return locations
  * provided.
  *
- * Returns: #FALSE on failure
+ * Returns: %FALSE on failure
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 
 gboolean
@@ -1041,14 +1541,16 @@ error:
 
 /**
  * dbus_g_type_struct_set:
- * @value: a #GValue containing a #DBusGTypeStruct type
+ * @value: a #GValue containing a struct instance
  * @member: struct member to set
  * @...: value for the first member, followed optionally by
- *       more member/value pairs, followed by #G_MAXUINT
+ *       more member/value pairs, followed by %G_MAXUINT
  *
  * Sets the selected members of the struct in @value.
  *
- * Returns: #FALSE on failure
+ * Returns: %FALSE on failure
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 
 gboolean
@@ -1213,6 +1715,8 @@ dbus_g_value_type_build_g_variant_type (GType type)
  * (including as a child type).
  *
  * Returns: a new #GVariant containing @value with a floating reference
+ *
+ * Deprecated: New code should use GDBus instead.
  */
 GVariant *
 dbus_g_value_build_g_variant (const GValue *value)
@@ -1320,9 +1824,15 @@ dbus_g_value_build_g_variant (const GValue *value)
   else if (type == G_TYPE_DOUBLE)
     return g_variant_new_double (g_value_get_double (value));
   else if (type == G_TYPE_STRING)
-    return g_variant_new_string (g_value_get_string (value));
+    {
+      const gchar *str = g_value_get_string (value);
+      return g_variant_new_string ((str != NULL) ? str : "");
+    }
   else if (type == G_TYPE_STRV)
-    return g_variant_new_strv (g_value_get_boxed (value), -1);
+    {
+      const gchar * const *strv = g_value_get_boxed (value);
+      return g_variant_new_strv (strv, (strv != NULL) ? -1 : 0);
+    }
   else if (type == DBUS_TYPE_G_OBJECT_PATH)
     return g_variant_new_object_path (g_value_get_boxed (value));
   else if (type == DBUS_TYPE_G_SIGNATURE)
