@@ -1,4 +1,8 @@
+/* SPDX-License-Identifier: AFL-2.1 OR GPL-2.0-or-later */
+
 #include <config.h>
+#undef G_DISABLE_ASSERT
+
 #include <string.h>
 #include <glib/gi18n.h>
 #include <glib-object.h>
@@ -659,6 +663,9 @@ gboolean
 my_object_echo_variant (MyObject *obj, GValue *variant, GValue *ret, GError **error)
 {
     GType t;
+
+    obj->echo_variant_called++;
+
     t = G_VALUE_TYPE(variant);
     g_value_init (ret, t);
     g_value_copy (variant, ret);
