@@ -3,6 +3,8 @@
  * Copyright © 2009 Collabora Ltd. <http://www.collabora.co.uk/>
  * Copyright © 2009-2011 Nokia Corporation
  *
+ * SPDX-License-Identifier: AFL-2.1 OR GPL-2.0-or-later OR MIT
+ *
  * In preparation for dbus-glib relicensing (if it ever happens), this file is
  * licensed under (at your option) either the AFL v2.1, the GPL v2 or later,
  * or an MIT/X11-style license:
@@ -36,6 +38,7 @@
  */
 
 #include <config.h>
+#undef G_DISABLE_ASSERT
 
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
@@ -54,10 +57,10 @@ setup (Fixture *f,
     gconstpointer path_to_use)
 {
   f->bus = dbus_g_bus_get_private (DBUS_BUS_SESSION, NULL, NULL);
-  g_assert (f->bus != NULL);
+  g_assert_nonnull (f->bus);
 
   f->bus2 = dbus_g_bus_get_private (DBUS_BUS_SESSION, NULL, NULL);
-  g_assert (f->bus2 != NULL);
+  g_assert_nonnull (f->bus2);
 }
 
 static void
