@@ -2,6 +2,8 @@
  *
  * Copyright © 2010 Collabora Ltd. <http://www.collabora.co.uk/>
  *
+ * SPDX-License-Identifier: AFL-2.1 OR GPL-2.0-or-later OR LGPL-2.1-or-later
+ *
  * Licensed under the Academic Free License version 2.1
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,6 +28,7 @@
  */
 
 #include <config.h>
+#undef G_DISABLE_ASSERT
 
 #include <dbus/dbus-glib.h>
 #include <gio/gio.h>
@@ -168,7 +171,7 @@ test_simple_equiv (void)
   v1 = g_variant_new_int32 (1984);
   v2 = g_variant_new_int32 (1984);
 
-  g_assert (test_g_variant_equivalent (v1, v2));
+  g_assert_true (test_g_variant_equivalent (v1, v2));
 
   g_variant_unref (v1);
   g_variant_unref (v2);
@@ -182,7 +185,7 @@ test_simple_not_equiv (void)
   v1 = g_variant_new_int32 (1982);
   v2 = g_variant_new_int32 (1984);
 
-  g_assert (!test_g_variant_equivalent (v1, v2));
+  g_assert_false (test_g_variant_equivalent (v1, v2));
 
   g_variant_unref (v1);
   g_variant_unref (v2);
@@ -207,7 +210,7 @@ test_array_not_equiv (void)
   g_variant_builder_add (&b, "v", g_variant_new_object_path ("/cats/escher"));
   v2 = g_variant_builder_end (&b);
 
-  g_assert (!test_g_variant_equivalent (v1, v2));
+  g_assert_false (test_g_variant_equivalent (v1, v2));
 
   g_variant_unref (v1);
   g_variant_unref (v2);
@@ -232,7 +235,7 @@ test_map_equiv (void)
   g_variant_builder_add (&b, "{os}", "/cats/josh", "Josh Smith");
   v2 = g_variant_builder_end (&b);
 
-  g_assert (test_g_variant_equivalent (v1, v2));
+  g_assert_true (test_g_variant_equivalent (v1, v2));
 
   g_variant_unref (v1);
   g_variant_unref (v2);
@@ -257,7 +260,7 @@ test_map_not_equiv1 (void)
   g_variant_builder_add (&b, "{os}", "/cats/rory", "Rory Cat");
   v2 = g_variant_builder_end (&b);
 
-  g_assert (!test_g_variant_equivalent (v1, v2));
+  g_assert_false (test_g_variant_equivalent (v1, v2));
 
   g_variant_unref (v1);
   g_variant_unref (v2);
@@ -281,7 +284,7 @@ test_map_not_equiv2 (void)
   g_variant_builder_add (&b, "{os}", "/cats/josh", "Josh Cat");
   v2 = g_variant_builder_end (&b);
 
-  g_assert (!test_g_variant_equivalent (v1, v2));
+  g_assert_false (test_g_variant_equivalent (v1, v2));
 
   g_variant_unref (v1);
   g_variant_unref (v2);
@@ -302,7 +305,7 @@ test_i (void)
 
   varc = g_variant_new_int32 (1984);
 
-  g_assert (test_g_variant_equivalent (var, varc));
+  g_assert_true (test_g_variant_equivalent (var, varc));
 
   g_variant_unref (var);
   g_variant_unref (varc);
@@ -322,7 +325,7 @@ test_s (void)
 
   varc = g_variant_new_string ("Orwell");
 
-  g_assert (test_g_variant_equivalent (var, varc));
+  g_assert_true (test_g_variant_equivalent (var, varc));
 
   g_variant_unref (var);
   g_variant_unref (varc);
@@ -342,7 +345,7 @@ test_o (void)
 
   varc = g_variant_new_object_path ("/cats/escher");
 
-  g_assert (test_g_variant_equivalent (var, varc));
+  g_assert_true (test_g_variant_equivalent (var, varc));
 
   g_variant_unref (var);
   g_variant_unref (varc);
@@ -370,7 +373,7 @@ test_us (void)
 
   varc = g_variant_new ("(us)", 1984, "Orwell");
 
-  g_assert (test_g_variant_equivalent (var, varc));
+  g_assert_true (test_g_variant_equivalent (var, varc));
 
   g_variant_unref (var);
   g_variant_unref (varc);
@@ -407,7 +410,7 @@ test_a_os (void)
   g_variant_builder_add (&b, "{os}", "/cats/josh", "Josh Smith");
   varc = g_variant_builder_end (&b);
 
-  g_assert (test_g_variant_equivalent (var, varc));
+  g_assert_true (test_g_variant_equivalent (var, varc));
 
   g_variant_unref (var);
   g_variant_unref (varc);
@@ -451,7 +454,7 @@ test_av (void)
   g_variant_builder_add (&b, "v", g_variant_new_object_path ("/cats/escher"));
   varc = g_variant_builder_end (&b);
 
-  g_assert (test_g_variant_equivalent (var, varc));
+  g_assert_true (test_g_variant_equivalent (var, varc));
 
   g_variant_unref (var);
   g_variant_unref (varc);
@@ -484,7 +487,7 @@ test_ab (void)
   g_variant_builder_add (&b, "b", FALSE);
   varc = g_variant_builder_end (&b);
 
-  g_assert (test_g_variant_equivalent (var, varc));
+  g_assert_true (test_g_variant_equivalent (var, varc));
 
   g_variant_unref (var);
   g_variant_unref (varc);
@@ -517,7 +520,7 @@ test_ai (void)
   g_variant_builder_add (&b, "i", 1066);
   varc = g_variant_builder_end (&b);
 
-  g_assert (test_g_variant_equivalent (var, varc));
+  g_assert_true (test_g_variant_equivalent (var, varc));
 
   g_variant_unref (var);
   g_variant_unref (varc);
@@ -550,7 +553,7 @@ test_au (void)
   g_variant_builder_add (&b, "u", 1066);
   varc = g_variant_builder_end (&b);
 
-  g_assert (test_g_variant_equivalent (var, varc));
+  g_assert_true (test_g_variant_equivalent (var, varc));
 
   g_variant_unref (var);
   g_variant_unref (varc);
@@ -590,7 +593,7 @@ test_ax (void)
   g_variant_builder_add (&b, "x", G_GINT64_CONSTANT (1066));
   varc = g_variant_builder_end (&b);
 
-  g_assert (test_g_variant_equivalent (var, varc));
+  g_assert_true (test_g_variant_equivalent (var, varc));
 
   g_variant_unref (var);
   g_variant_unref (varc);
@@ -629,7 +632,7 @@ test_at (void)
   g_variant_builder_add (&b, "t", G_GUINT64_CONSTANT (1066));
   varc = g_variant_builder_end (&b);
 
-  g_assert (test_g_variant_equivalent (var, varc));
+  g_assert_true (test_g_variant_equivalent (var, varc));
 
   g_variant_unref (var);
   g_variant_unref (varc);
@@ -662,7 +665,7 @@ test_ay (void)
   g_variant_builder_add (&b, "y", 42);
   varc = g_variant_builder_end (&b);
 
-  g_assert (test_g_variant_equivalent (var, varc));
+  g_assert_true (test_g_variant_equivalent (var, varc));
 
   g_variant_unref (var);
   g_variant_unref (varc);
@@ -682,7 +685,7 @@ test_g (void)
 
   varc = g_variant_new_signature ("a{u(ua{sa{sv}})}");
 
-  g_assert (test_g_variant_equivalent (var, varc));
+  g_assert_true (test_g_variant_equivalent (var, varc));
 
   g_variant_unref (var);
   g_variant_unref (varc);
