@@ -3,6 +3,8 @@
  *
  * Copyright (C) 2003, 2004  Red Hat Inc.
  *
+ * SPDX-License-Identifier: AFL-2.1 OR GPL-2.0-or-later
+ *
  * Licensed under the Academic Free License version 2.1
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -611,7 +613,7 @@ read_and_drop_on_floor (int fd,
   char *allocated;
   char not_allocated[512+PAYLOAD_SIZE];
 
-  g_assert (count < (int) sizeof(not_allocated));
+  g_assert_cmpint (count, <, (int) sizeof (not_allocated));
   
   if (fake_malloc_overhead)
     {
@@ -669,7 +671,7 @@ write_junk (int fd,
   char *allocated;
   char not_allocated[512+PAYLOAD_SIZE] = { '\0', };
 
-  g_assert (count < (int) sizeof(not_allocated));
+  g_assert_cmpint (count, <, (int) sizeof (not_allocated));
   
   if (fake_malloc_overhead)
     {
@@ -1095,7 +1097,6 @@ main (int argc, char *argv[])
 #endif
 
 #ifndef TEST_PROFILE_DISABLED
-  g_thread_init (NULL);
   dbus_g_thread_init ();
 
 #ifndef G_DISABLE_ASSERT
